###############################################################################
#
# Copyright (c) 2011 Projekt01 GmbH and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
###############################################################################
"""
$Id:$
"""
__docformat__ = "reStructuredText"

import zope.interface
import zope.publisher.interfaces.http

import m01.gmap.util

from m01.gmap import interfaces


class GMapAPIProviderBase(object):
    """Gmap api provider base class"""

    zope.component.adapts(zope.interface.Interface,
        zope.publisher.interfaces.http.IHTTPRequest,
        zope.interface.Interface)

    def __init__(self, context, request, view):
        self.__parent__ = view
        self.context = context
        self.request = request

    def update(self):
        pass


@zope.interface.implementer(interfaces.IGMapAPIProvider)
class GMapAPIProvider(GMapAPIProviderBase):
    """Gmap api provider.

    Use this GMap api key provider in your pagetemplate like:
    
    <tal:block replace="structure provider:IGMapAPIProvider"> </tal:block>
    
    """

    def render(self):
        return m01.gmap.util.GMAP_HTTPS_JAVASCRIPT


@zope.interface.implementer(interfaces.IHTTPSGMapAPIProvider)
class HTTPSGMapAPIProvider(GMapAPIProviderBase):
    """Gmap api provider supporting https API call.

    Use this GMap api key provider in your pagetemplate like:
    
    <tal:block replace="structure provider:IHTTPSGMapAPIProvider"> </tal:block>
    
    """

    def render(self):
        return m01.gmap.util.GMAP_HTTPS_JAVASCRIPT



###############################################################################
#
# Copyright (c) 2011 Projekt01 GmbH and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
###############################################################################
"""
$Id:$
"""
__docformat__ = "reStructuredText"

import zope.interface
import zope.i18n
import zope.i18nmessageid
from zope.traversing.browser import absoluteURL
from zope.component import hooks

from z3c.form import widget

from z3c.form.interfaces import NOVALUE
from z3c.form import widget
from z3c.form import validator
from z3c.form import converter
from z3c.form.browser import text

from m01.gmap import interfaces
from m01.gmap import schema

try:
    unicode         # p01.checker.silence
except Exception:
    unicode = str

_ = zope.i18nmessageid.MessageFactory('p01')


m01_gmap_template = """<script type="text/javascript">
  $("#%s").m01GMapWidget({%s
  });
</script>
"""

def m01GMapWidgetJavaScript(m01GMapExpression, data):
    """GMap JavaScript generator.

    Knows how to generate customization for m01GMap JS.
    This is usefull for generate custom image path.

    The data argument allows you to set key, value pairs for customized data.
    """

    lines = []
    append = lines.append
    for key, value in data.items():
        if key == 'mapType':
            if value is None:
                # skip empty mapType, use script defaults
                continue
            append('\n    mapType: %s' % value)
        elif value is True:
            append('\n    %s: true' % key)
        elif value is False:
            append('\n    %s: false' % key)
        elif value is None:
            append('\n    %s: null' % key)
        elif isinstance(value, int):
            append('\n    %s: %s' % (key, value))
        elif isinstance(value, (str, unicode)):
            if value.startswith('$'):
                append('\n    %s: %s' % (key, value))
            else:
                append('\n    %s: "%s"' % (key, value))
        else:
            append('\n    %s: %s' % (key, value))
    options = ','.join(lines)

    return m01_gmap_template % (m01GMapExpression, options)


class GMapWidgetBase(text.TextWidget):
    """Generic google map widget"""

    _siteURL = None

    # widget options
    css = u'm01-gmap'

    # map config
    mapType = None
    width = 400
    height = 300
    responsive = True
    zoom = 11

    # info window content for display widget mode
    infoWindowMessageDisplayMode = u''

    # info window content for input widget mode
    infoWindowMessageInputMode = _(
        u'Drag and drop the marker and save the form. <br />'
        u'Double click the marker for remove them.')

    # marker icon config
    iconName = 'm01GMapWidgetIcon.png'
    iconShadowName = 'm01GMapWidgetIconShadow.png'
    iconWidth = 19
    iconHeight = 32
    iconAnchorXOffset = 9
    iconAnchorYOffset = 30

    # info window config
    infoWindowAnchorXOffset = 10
    infoWindowAnchorYOffset = 0

    latitudeFallback = 10
    longitudeFallback = 10
    zoomFallback = 4

    # enable remove geo location
    removeMarkerOnDBClick = True

    # widget update will override this value
    value = {'lat': u'', 'lon': u''}

    @property
    def style(self):
        return 'width: %spx; height: %spx' % (self.width, self.height)

    @property
    def siteURL(self):
        if self._siteURL is None:
            site = hooks.getSite()
            self._siteURL = absoluteURL(site, self.request)
        return self._siteURL

    @property
    def iconURL(self):
        return '%s/@@/%s' % (self.siteURL, self.iconName)

    @property
    def iconShadowURL(self):
        if self.iconShadowName is not None:
            return '%s/@@/%s' % (self.siteURL, self.iconShadowName)

    @property
    def infoWindowContent(self):
        if self.mode == 'display':
            msg = self.infoWindowMessageDisplayMode
            return zope.i18n.translate(msg, context=self.request)
        if self.mode == 'input':
            msg = self.infoWindowMessageInputMode
            return zope.i18n.translate(msg, context=self.request)
        return u''

    @property
    def address(self):
        """Simple address lookup.

        Note that this implementation requires the following attributes given
        from the context:

        street, city, state, zip, country

        The result will be in the US format which fits for other countries too
        even if the format is not 100% correct. The generated address looks
        like:

        street, city, state zip, country

        Note, the state and zip do not contain a comma. Also note that the
        country must be a real country name and not a iso country code.

        If this doesn't fit, feel free to implement your own configuration
        and register them as IGMapWidgetConfiguration adapter with
        (context, request, gmap or None, view, field, widget) as discriminators.
        """
        args = []
        append = args.append
        if self.context is not None:
            # plain widget setup doesn't provide a context
            if self.context.street:
                append(self.context.street)
            if self.context.city:
                append(self.context.city)
            if self.context.state and self.context.zip:
                append('%s %s' % (self.context.state, self.context.zip))
            elif self.context.state and not self.context.zip:
                append(self.context.state)
            elif not self.context.state and self.context.zip:
                append(self.context.zip)
            if self.context.country:
                append(self.context.country)
        return ', '.join(args)

    @property
    def names(self):
        return {'lat': '%s-latitude' % self.name,
                'lon': '%s-longitude' % self.name}

    @property
    def ids(self):
        return {'lat': '%s-latitude' % self.id,
                'lon': '%s-longitude' % self.id}

    def extract(self, default=NOVALUE, setErrors=True):
        """See z3c.form.interfaces.IWidget."""
        value = {}
        for key, name in self.names.items():
            val = self.request.get(name, None)
            if val is not None:
                value[key] = val
        return value or NOVALUE

    @property
    def gMapJavaScript(self):
        latId = self.ids['lat']
        lngId = self.ids['lon']
        latitudeExpression = '#%s' % latId.replace('.', '\\\.')
        longitudeExpression = '#%s' % lngId.replace('.', '\\\.')
        data = {
            # internal generated widget values
            'mode': self.mode,
            'infoWindowContent': self.infoWindowContent,
            'latitude': self.value.get('lat') or None,
            'longitude': self.value.get('lon') or None,
            'latitudeFallback': self.latitudeFallback,
            'longitudeFallback': self.longitudeFallback,
            'latitudeExpression': latitudeExpression,
            'longitudeExpression': longitudeExpression,
            'address': self.address,
            'zoom': self.zoom,
            'zoomFallback': self.zoomFallback,
            'iconURL': self.iconURL,
            'iconWidth': self.iconWidth,
            'iconHeight': self.iconHeight,
            'iconAnchorXOffset': self.iconAnchorXOffset,
            'iconAnchorYOffset': self.iconAnchorYOffset,
            'infoWindowAnchorXOffset': self.infoWindowAnchorXOffset,
            'infoWindowAnchorYOffset': self.infoWindowAnchorYOffset,
            'responsive': self.responsive,
           }
        if self.mapType is not None:
            data['mapType'] = self.mapType
        if self.iconShadowName is not None:
            data['iconShadowURL'] = self.iconShadowURL
        m01GMapWidgetExpression = self.id.replace('.', '\\\.')
        return m01GMapWidgetJavaScript(m01GMapWidgetExpression, data)


@zope.interface.implementer_only(interfaces.IGMapWidget)
class GMapWidget(GMapWidgetBase):
    """Generic google map widget"""


def getGMapWidget(field, request):
    """IFieldWidget factory for GMapWidget."""
    # add gmap widget marker to annotations
    request.annotations['m01.gmap.widget.GMapWidget'] = True
    return widget.FieldWidget(field, GMapWidget(request))


@zope.interface.implementer_only(interfaces.IGeoPointGMapWidget)
class GeoPointGMapWidget(GMapWidgetBase):
    """GeoPoint google map widget"""


def getGeoPointGMapWidget(field, request):
    """IFieldWidget factory for GeoPointGMapWidget."""
    # add gmap widget marker to annotations
    request.annotations['m01.gmap.widget.GMapWidget'] = True
    return widget.FieldWidget(field, GeoPointGMapWidget(request))



###############################################################################
#
# Copyright (c) 2011 Projekt01 GmbH and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
###############################################################################
"""
$Id:$
"""
__docformat__ = "reStructuredText"

import zope.schema
import zope.contentprovider.interfaces

from z3c.form.interfaces import IWidget


class IGMapAPIProvider(zope.contentprovider.interfaces.IContentProvider):
    """Gmap api javascript provider interface"""


class IHTTPSGMapAPIProvider(IGMapAPIProvider):
    """Gmap api javascript provider interface supporting https"""


class IGMapWidgetSchema(IWidget):
    """Google map widget schema."""

    # implement your own address lookup if the default street, city etc.
    # doesn't fit
    address = zope.schema.TextLine(
        title=u'Google Map address',
        description=u'The google map geo location address string.',
        default=None,
        missing_value=u'',
        required=True)

    mapType = zope.schema.Choice(
        title=u'Google map type',
        description=u'The google map type',
        values=[u'G_NORMAL_MAP', u'G_SATELLITE_MAP', u'G_HYBRID_MAP'],
        default=u'G_NORMAL_MAP',
        required=True)

    width = zope.schema.Int(
        title=u'Map width',
        description=u'The map width in pixle',
        default=500,
        required=True)

    height = zope.schema.Int(
        title=u'Map height in pixle',
        description=u'The map height in pixle',
        default=400,
        required=True)

    iconName = zope.schema.TextLine(
        title=u'Map marker icon name',
        description=u'The map marker icon name',
        required=True)

    iconShadowName = zope.schema.TextLine(
        title=u'Map marker icon shadow name',
        description=u'The map marker icon shadow name',
        required=True)

    iconURL = zope.schema.URI(
        title=u'Map marker icon url',
        description=u'The map marker icon url',
        required=True)

    iconShadowURL = zope.schema.URI(
        title=u'Map marker icon shadow url',
        description=u'The map marker icon url',
        required=True)

    iconWidth = zope.schema.Int(
        title=u'Map marker icon width in pixle',
        description=u'The map marker icon width in pixle',
        default=19,
        required=True)

    iconHeight = zope.schema.Int(
        title=u'Map marker icon height',
        description=u'The map marker icon height',
        default=32,
        required=True)

    iconAnchorXOffset = zope.schema.Int(
        title=u'Icon anchor offset x',
        description=u'The icon anchor offset x',
        default=9,
        required=True)

    iconAnchorYOffset = zope.schema.Int(
        title=u'Icon anchor offset y',
        description=u'The icon anchor offset y',
        default=30,
        required=True)

    infoWindowAnchorXOffset = zope.schema.Int(
        title=u'Icon window anchor offset x',
        description=u'The icon window anchor offset x',
        default=10,
        required=True)

    infoWindowAnchorYOffset = zope.schema.Int(
        title=u'Icon window anchor offset y',
        description=u'The icon window anchor offset y',
        default=0,
        required=True)

    zoom = zope.schema.Int(
        title=u'Zoom factor',
        description=u'The map zoom factor',
        default=10,
        required=True)

    infoWindowContent = zope.schema.TextLine(
        title=u'Message used for info window',
        description=u'The message used for info window',
        default=u'',
        required=True)

    infoWindowMessageDisplayMode = zope.schema.TextLine(
        title=u'Content shown in google map info window for display mode',
        description=u'The info window content for display widget mode. If an '
                      'empty string is used no info window is shown',
        default=u'',
        required=True)

    infoWindowMessageInputMode = zope.schema.TextLine(
        title=u'Content shown in google map info window for input mode',
        description=u'The content get used in the google map info window popup.',
        default=u'',
        required=True)


class IGMapWidget(IGMapWidgetSchema):
    """GeoLocation GMapWidget schema"""


class IGeoPointGMapWidget(IGMapWidgetSchema):
    """GeoPoint GMapWidget schema"""



###############################################################################
#
# Copyright (c) 2011 Projekt01 GmbH and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
###############################################################################
"""
$Id:$
"""
__docformat__ = "reStructuredText"

try:
    unicode         # p01.checker.silence
except Exception as e:
    unicode = str

import zope.schema
import zope.component

from z3c.form import converter

import  m01.mongo.geo

from m01.gmap import interfaces


class GMapWidgetDataConverterBase(converter.BaseDataConverter):
    """GMapWidget data converter base class."""

    def __init__(self, field, widget):
        self.field = field
        self.widget = widget

    def toWidgetValue(self, value):
        """See interfaces.IDataConverter"""
        if value is None:
            lon = ''
            lat = ''
        elif isinstance(value, dict):
            if value.get('coordinates'):
                # GeoPoint
                coordinates = value.get('coordinates')
                if coordinates and len(coordinates) == 2:
                    # set lon, lat
                    lon = coordinates[0]
                    lat = coordinates[1]
            else:
                # GeoLocation
                lon = value.get('lon')
                lat = value.get('lat')
        elif isinstance(value, (list, tuple)):
            lon = value[0]
            lat = value[1]
        else:
            lon = value.lon
            lat = value.lat
        return {'lon' : lon, 'lat': lat}

    def toFieldValue(self, value):
        """See interfaces.IDataConverter"""
        lon = value.get('lon')
        lat = value.get('lat')
        if lon and lat:
            geoLocation = self.field.get(self.widget.context)
            if geoLocation is None:
                geoLocation = self.geoObjectFactory({'lon':lon, 'lat': lat})
                geoLocation.__parent__ = self.widget.context
                geoLocation.__name__ = str(self.field.__name__)
            else:
                # just set the values, we will prevent write access in the
                # GeoLocation implementation itself if the value is the same
                geoLocation.lon = float(lon)
                geoLocation.lat = float(lat)
            return geoLocation
        else:
            return None


class GMapWidgetDataConverter(GMapWidgetDataConverterBase):
    """GMapWidget data converter for IGeoLocation objects."""

    zope.component.adapts(zope.schema.Object,
        interfaces.IGMapWidget)

    geoObjectFactory = m01.mongo.geo.GeoLocation


class GeoPointGMapWidgetDataConverter(GMapWidgetDataConverterBase):
    """GeoPointGMapWidget data converter for IGeoPoint objects."""

    zope.component.adapts(zope.schema.Object,
        interfaces.IGeoPointGMapWidget)

    geoObjectFactory = m01.mongo.geo.GeoPoint



###############################################################################
#
# Copyright (c) 2011 Projekt01 GmbH and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
###############################################################################
"""
$Id:$
"""
__docformat__ = "reStructuredText"

import os

from zope.app.appsetup.product import getProductConfiguration

try:
    unicode         # p01.checker.silence
except Exception as e:
    unicode = str


JAVASCRIPT_TEMPLATE = """<script type="text/javascript" src="%s"> </script>"""


def getConfig(product, confKey, envKey, required=True):
    """Get product configuration based on zope.app.appsetup or env variable."""
    value = None
    config = getProductConfiguration(product)
    if config is not None:
        value = config.get(confKey)
    else:
        value = os.environ.get(envKey)
    if value is None and required:
        raise ValueError(
            "You must define '%s' product config '%s' in buildout.cfg" % (
                confKey, product))
    if value:
        return unicode(value)


# Google Map API key loaded from m01.gmap product config or env variable
GMAP_API_KEY = getConfig('m01.gmap', 'key', 'M01_GMAP_API_KEY', required=False)


def getGMapJavaScript(apiKey, protocol='http'):
    """Google Map API key based on M01_GMAP_API_KEY value"""
    if protocol:
        protocol += ':'
        protocol = protocol.replace('::', ':')
    if apiKey is None:
        return ''
    srcURL = '%s//maps.google.com/maps?file=api&amp;v=2&amp;key=%s' % (protocol,
        apiKey)
    return JAVASCRIPT_TEMPLATE % srcURL

# GMAP JavaScript
GMAP_JAVASCRIPT = getGMapJavaScript(GMAP_API_KEY, '')
GMAP_HTTP_JAVASCRIPT = getGMapJavaScript(GMAP_API_KEY, 'http')
GMAP_HTTPS_JAVASCRIPT = getGMapJavaScript(GMAP_API_KEY, 'https')



##############################################################################
#
# Copyright (c) 2011 Projekt01 GmbH and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""
$Id:$
"""

import os
from setuptools import setup, find_packages

def read(*rnames):
    return open(os.path.join(os.path.dirname(__file__), *rnames)).read()

setup (
    name='m01.gmap',
    version='0.8.3',
    author = "Roger Ineichen, Projekt01 GmbH",
    author_email = "dev@projekt01.ch",
    description = "Google map geo location and z3c.form widget for Zope3",
    long_description=(
        read('README.txt')
        + '\n\n' +
        read('src', 'm01', 'gmap', 'README.txt')
        + '\n\n' +
        read('CHANGES.txt')
        ),
    long_description_content_type = 'text/plain',
    license = "ZPL 2.1",
    keywords = "zope3 z3c form google map widget m01",
    classifiers = [
        'Development Status :: 4 - Beta',
        'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: Zope Public License',
        'Programming Language :: Python',
        'Natural Language :: English',
        'Operating System :: OS Independent',
        'Topic :: Internet :: WWW/HTTP',
        'Framework :: Zope3'],
    url = 'http://pypi.python.org/pypi/m01.gmap',
    packages = find_packages('src'),
    include_package_data = True,
    package_dir = {'':'src'},
    namespace_packages = ['m01'],
    extras_require=dict(
        test=[
            'zope.testing',
            'p01.checker',
            'z3c.form',
             ]),
    install_requires = [
        'setuptools',
        'm01.mongo',
        'z3c.form',
        'zope.app.appsetup',
        'zope.component',
        'zope.contentprovider',
        'zope.i18n',
        'zope.i18nmessageid',
        'zope.interface',
        'zope.publisher',
        'zope.schema',
        'zope.traversing',
        ],
      zip_safe = False,
)


###############################################################################
#
# Copyright (c) 2011 Projekt01 GmbH and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
###############################################################################
"""
$Id:$
"""
__docformat__ = "reStructuredText"

import zope.interface
import zope.schema


class IGMapObject(zope.schema.interfaces.IObject):
    """Google map field."""


@zope.interface.implementer(IGMapObject)
class GMapObject(zope.schema.Object):
    """Google map object field."""

    def __init__(self, **kw):
        schema = kw.pop('schema')
        super(GMapObject, self).__init__(schema, **kw)
